#ifndef AudioDeviceSocket_h
#define AudioDeviceSocket_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "ObjectDeleter.h"

#include <QObject>
#include <QTcpSocket>

#include <memory>

class AudioDeviceInterface;

//* audio device socket to read request and write stream
class AudioDeviceSocket: public QObject, private Base::Counter<AudioDeviceSocket>
{

    Q_OBJECT

    public:

    //* constructor
    explicit AudioDeviceSocket( AudioDeviceInterface*, QTcpSocket* );

    private Q_SLOTS:

    //* read from socket
    void _read();

    private:

    //* check device interface
    bool _checkDeviceInterface() const;

    //* write mimetype
    void _writeMimeType( const QString&, qint32 );

    //* wav header
    void _writeWavHeader( qint32 );

    //* device interface
    AudioDeviceInterface* interface_ = nullptr;

    //* socket
    std::unique_ptr<QTcpSocket, Base::ObjectDeleter> socket_;

};

#endif
