/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CollectionItem.h"

//__________________________________________________________________
const QString CollectionItem::MimeType = "musicbox/collection-item-list";

//__________________________________________________________________
CollectionItem::Type CollectionItem::parentType( CollectionItem::Type type )
{
    switch( type )
    {
        default:
        case Type::Invalid:
        case Type::Artist:
        return Type::Invalid;

        case Type::Album: return Type::Artist;
        case Type::Track: return Type::Album;
    }
}

//__________________________________________________________________
bool CollectionItem::isChild( const CollectionItem& other ) const
{

    switch( type_ )
    {
        default:
        case Type::Invalid:
        case Type::Artist: return false;
        case Type::Album: return other.type_ == Type::Artist && other.artist() == artist();
        case Type::Track: return other.type_ == Type::Album && other.artist() == artist() && other.album() == album() && other.disc() == disc();
    }

}

//__________________________________________________________________
bool operator == (const CollectionItem& lhs, const CollectionItem& rhs )
{
    if( lhs.type_ != rhs.type_ ) return false;
    else switch( lhs.type_ )
    {
        default: return false;
        case CollectionItem::Type::Artist: return lhs.artist() == rhs.artist();
        case CollectionItem::Type::Album: return lhs.artist() == rhs.artist() && lhs.album() == rhs.album() && lhs.disc() == rhs.disc();
        case CollectionItem::Type::Track: return lhs.artist() == rhs.artist() && lhs.album() == rhs.album() && lhs.disc() == rhs.disc() && lhs.titleOrFile() == rhs.titleOrFile();
    }
}

//__________________________________________________________________
bool operator < (const CollectionItem& lhs, const CollectionItem& rhs )
{
    if( lhs.type() != rhs.type() )
    {

        if( lhs.type() == CollectionItem::Type::Track ) return true;
        else if( lhs.type() == CollectionItem::Type::Album ) return rhs.type() == CollectionItem::Type::Artist;
        else return false;

    } else  switch( lhs.type_ ) {

        default: return false;

        case CollectionItem::Type::Artist: return lhs.shortArtist() < rhs.shortArtist();
        case CollectionItem::Type::Album:
        {
            if( lhs.artist() != rhs.artist() ) return lhs.artist() < rhs.artist();
            else if( lhs.album() == rhs.album() )
            {
                if( lhs.disc() >= 0 && rhs.disc() >= 0 ) return lhs.disc() < rhs.disc();
                else if( lhs.disc() >= 0 ) return false;
                else if( rhs.disc() >= 0 ) return true;
                else return false;

            } else if( lhs.year() != rhs.year() ) {

                if( lhs.year() >= 0 && rhs.year() >= 0 ) return lhs.year() < rhs.year();
                else if( lhs.year() >= 0 ) return false;
                else if( rhs.year() >= 0 ) return true;
                else return false;
            }
            else return lhs.album() < rhs.album();
        }

        case CollectionItem::Type::Track:
        {
            if( lhs.artist() != rhs.artist() ) return lhs.artist() < rhs.artist();
            else if( lhs.album() != rhs.album() ) return lhs.album() < rhs.album();
            else if( lhs.disc() != rhs.disc() )
            {
                if( lhs.disc() >= 0 && rhs.disc() >= 0 ) return lhs.disc() < rhs.disc();
                else if( lhs.disc() >= 0 ) return false;
                else if( rhs.disc() >= 0 ) return true;
                else return false;
            }
            else if( lhs.track() != rhs.track() )
            {
                if( lhs.track() >= 0 && rhs.track() >= 0 ) return lhs.track() < rhs.track();
                else if( lhs.track() >= 0 ) return false;
                else if( rhs.track() >= 0 ) return true;
                else return false;
            }
            else return lhs.titleOrFile() < rhs.titleOrFile();
        }
    }

}
