#ifndef CollectionModel_h
#define CollectionModel_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "CollectionItem.h"
#include "CoverCache.h"
#include "TreeModel.h"

//* collection model
class CollectionModel: public TreeModel<CollectionItem>, private Base::Counter<CollectionModel>
{

    Q_OBJECT

    public:

    //* constructor
    explicit CollectionModel( QObject* = nullptr );

    //* colums
    enum ColumnType
    {
        Name,
        nColumns
    };

    //*@name accessors
    //@{

    //* flags
    Qt::ItemFlags flags( const QModelIndex& ) const override;

    //* mime data
    QMimeData* mimeData( const QModelIndexList& ) const override;

    //* data
    QVariant data( const QModelIndex&, int ) const override;

    //* children
    QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole ) const override;

    //* columns
    int columnCount(const QModelIndex& = QModelIndex()) const override
    { return nColumns; }

    //* children
    /** needs overwrite because of lazy filling of the list */
    bool hasChildren( const QModelIndex& ) const override;

    //* get icon from collection item
    QIcon icon( const CollectionItem& ) const;

    //@}

    //*@name modifiers
    //@{

    //* update artist
    /* copy number of tracks and total length from argument to tree copy */
    bool updateArtist( const CollectionItem& );

    //* update album
    /* copy number of tracks and total length from argument to tree copy */
    bool updateAlbum( const CollectionItem& );

    //* cover cache
    void setCoverCache( const CoverCache::Pointer& );

    //@}

    private Q_SLOTS:

    //* configuration
    void _updateConfiguration();

    private:

    //* cover cache
    CoverCache::Pointer cache_;

    //* show artist icon
    bool showArtistIcon_ = false;

};


#endif
