#ifndef CurrentTrackWidget_h
#define CurrentTrackWidget_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "MpdDefines.h"

#include <QBasicTimer>
#include <QLabel>
#include <QSlider>
#include <QTimerEvent>
#include <QWidget>

class CurrentTrackWidget: public QWidget, private Base::Counter<CurrentTrackWidget>
{

    Q_OBJECT

    public:

    //* constructor
    explicit CurrentTrackWidget( QWidget* = nullptr );

    //*@name accessors
    //@{

    //* elapsed time (seconds)
    quint32 elapsedTime() const
    { return elapsedTime_/precision_; }

    //* total time (seconds)
    quint32 totalTime() const
    { return totalTime_/precision_; }

    //@}

    //*@name modifiers
    //@{

    //* clear
    void clear();

    //* set state
    void setState( Mpd::State );

    //* total time (seconds)
    void setTotalTime( quint32 );

    //* elapsed time (seconds)
    void setElapsedTime( quint32 );

    //* enable seek
    void setSeekEnabled( bool );

    //* start timer
    void startTimer();

    //* stop timer
    void stopTimer();

    //@}

    Q_SIGNALS:

    //* request seek
    void requestSeek( qreal );

    public Q_SLOTS:

    //* change orientation
    void setOrientation( const Qt::Orientation& );

    protected:

    //* timer event
    void timerEvent( QTimerEvent* ) override;

    private Q_SLOTS:

    //* slider moved
    void _processSliderMoved();

    //* slider released
    void _processSliderReleased();

    private:

    //* update time labels
    void _updateLabels();

    //* update slider
    void _updateSlider();

    //* elapsed time label
    QLabel* elapsedTimeLabel_ = nullptr;

    //* remaining time label
    QLabel* remainingTimeLabel_ = nullptr;

    //* slider
    QSlider* slider_ = nullptr;

    //* state
    Mpd::State state_ = Mpd::State::Stopped;

    //* precision
    static const int precision_;

    //* elapsed time
    quint32 elapsedTime_ = 0;

    //* total time
    quint32 totalTime_ = 0;

    //* timer
    QBasicTimer timer_;

};

#endif
