#ifndef PlayerAdaptor_h
#define PlayerAdaptor_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "MainWindow.h"
#include "TrackInformation.h"

#include <QDBusAbstractAdaptor>
#include <QDBusObjectPath>

namespace DBus
{

    /** see: https://specifications.freedesktop.org/mpris-spec/latest/Player_Interface.html */
    class PlayerAdaptor: public QDBusAbstractAdaptor, private Base::Counter<PlayerAdaptor>
    {

        Q_OBJECT
        Q_CLASSINFO("D-Bus Interface", "org.mpris.MediaPlayer2.Player")

        public:

        //* constructor
        explicit PlayerAdaptor( MainWindow* );

        //* parent
        inline MainWindow* parent() const
        { return static_cast<MainWindow*>( QObject::parent() ); }

        Q_PROPERTY( bool CanControl READ canControl )
        bool canControl() const
        { return true; }

        Q_PROPERTY( bool CanGoNext READ canGoNext )
        bool canGoNext() const;

        Q_PROPERTY( bool CanGoPrevious READ canGoPrevious )
        bool canGoPrevious() const;

        Q_PROPERTY( bool CanPause READ canPause )
        bool canPause() const;

        Q_PROPERTY( bool CanPlay READ canPlay )
        bool canPlay() const;

        Q_PROPERTY( bool CanSeek READ canSeek )
        bool canSeek() const;

        Q_PROPERTY( QString LoopStatus READ loopStatus WRITE setLoopStatus )
        QString loopStatus() const;
        void setLoopStatus( const QString& );

        Q_PROPERTY( double MaximumRate READ maximumRate )
        double maximumRate() const { return 1; }

        Q_PROPERTY( double MinimumRate READ minimumRate )
        double minimumRate() const { return 1; }

        Q_PROPERTY( QVariantMap Metadata READ metadata )
        QVariantMap metadata() const;

        Q_PROPERTY( QString PlaybackStatus READ playbackStatus )
        QString playbackStatus() const;

        Q_PROPERTY( qlonglong Position READ position )
        qlonglong position() const;

        Q_PROPERTY( double Rate READ rate WRITE setRate )
        double rate() const
        { return 1; }

        void setRate(double value)
        {}

        Q_PROPERTY( bool Shuffle READ shuffle WRITE setShuffle )
        bool shuffle() const;

        void setShuffle( bool );

        Q_PROPERTY( double Volume READ volume WRITE setVolume)
        double volume() const;

        void setVolume(double value);

        public Q_SLOTS:

        //* previous track
        void Previous();

        //* next track
        void Next();

        //* play
        void Play();

        //* pause
        void Pause();

        //* toggle play/pause
        void PlayPause();

        //* stop
        void Stop();

        //* advance by position
        void Seek(qlonglong Offset);

        //* set position
        void SetPosition(const QDBusObjectPath &TrackId, qlonglong Position);

        //* open arbitrary file
        void OpenUri(const QString&)
        {}

        Q_SIGNALS: // SIGNALS

        void Seeked(qlonglong Position);

        private Q_SLOTS:

        //* process status update
        void _updateStatus( MainWindow::StatusFlags );

        private:

        //* current track id
        QString _trackId( const TrackInformation& ) const;

        //* signal property changed
        void _signalPropertyChanged( const QString&, const QVariant& );
        void _signalPropertyChanged( const QVariantMap& );

    };

}
#endif
