#ifndef LyricsProvider_h
#define LyricsProvider_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseLyricsProvider.h"

#include <QDomElement>
#include <QString>
#include <QStringList>
#include <QRegExp>

class LyricsProvider: public BaseLyricsProvider
{

    Q_OBJECT

    public:

    //* constructor
    explicit LyricsProvider( QObject* );

    //* from dom element
    explicit LyricsProvider( QObject*, const QDomElement& );

    //*@name modifiers
    //@{

    //* url
    void setUrl( const QString& value )
    { url_ = value; }

    //* add conversion
    void addConversion( const QRegExp& begin, const QString& end )
    { conversionList_.append( { begin, end } ); }

    //* add extraction tags
    void addExtractionTags( const QString& begin, const QString& end = QString() )
    { extractionTags_.append( { begin, end } ); }

    //* add exclusion tags
    void addExclusionTags( const QString& begin, const QString& end = QString() )
    { exclusionTags_.append( { begin, end } ); }

    //@}

    public Q_SLOTS:

    //* get lyrics for given track
    void get( const TrackInformation& ) override;

    private Q_SLOTS:

    //* process lyrics
    void _processLyrics( const TrackInformation&, const QByteArray& );

    private:

    //* add conversion
    void _addConversion( const QDomElement& );

    //* add extraction tags
    void _addExtractionTags( const QDomElement& );

    //* add exclusion tags
    void _addExclusionTags( const QDomElement& );

    //* add invalid indicator
    void _addInvalidIndicator( const QDomElement& );

    //* replace track information base tags
    QString _replaceTags( const QString&, const TrackInformation& ) const;

    //* convert
    QString _convert( const QString& ) const;

    //* url string
    QString url_;

    //* charset
    QString charset_;

    //* convertion rules
    using ConversionPair = QPair<QRegExp, QString>;
    using ConversionList = QList<ConversionPair>;
    ConversionList conversionList_;

    //* extraction tags
    using TagPair=QPair<QString, QString>;
    using TagList = QList<TagPair>;
    TagList extractionTags_;

    //* exclusion tags
    TagList exclusionTags_;

    //* invalid indicator
    QStringList invalidIndicatorList_;

};

#endif
