#ifndef MpdLocalSocket_h
#define MpdLocalSocket_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MpdBaseSocket.h"
#include "Counter.h"

#include <QLocalSocket>
#include <memory>

namespace Mpd
{

    //* local socket implementation
    class LocalSocket: public BaseSocket, private Base::Counter<LocalSocket>
    {
        Q_OBJECT

        public:

        //* constructor
        explicit LocalSocket( QObject* );

        //*@name accessors
        //@{

        //* error string
        QString errorString() const override
        { return socket_->errorString(); }

        //* bytes available
        qint64 bytesAvailable() const override
        { return socket_->bytesAvailable(); }

        //* true if connected
        bool isConnected() const override
        { return socket_->state() == QLocalSocket::ConnectedState; }

        //@}

        //*@name modifiers
        //@{

        using QObject::connect;

        //* connect to host defined by server configuration
        /** this is blocking */
        bool connect( const ServerConfiguration& ) override;

        // disconnect
        /** this is blocking */
        void disconnect() override;

        //* read
        QByteArray read( qint64 maxSize ) override
        { return socket_->read( maxSize ); }

        //* read all
        QByteArray readAll() override
        { return socket_->readAll(); }

        //* write
        qint64 write( const char* data ) override
        { return socket_->write( data ); }

        //* wait for ready read
        bool waitForReadyRead() override
        { return socket_->waitForReadyRead(); }

        //* close
        void close() override
        { socket_->close(); }

        //@}

        private:

        //* socket
        QLocalSocket* socket_;

    };

}

#endif
