#ifndef PlaylistFindWidget_h
#define PlaylistFindWidget_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AbstractFindWidget.h"

#include "Counter.h"
#include "LineEditor.h"

class PlaylistFindWidget: public AbstractFindWidget, private Base::Counter<PlaylistFindWidget>
{

    Q_OBJECT

    public:

    //* constructor
    explicit PlaylistFindWidget( QWidget* = nullptr );

    //*@name accessors
    //@{

    //* string to find
    QString text() const override
    { return editor_->text(); }

    //* get selection
    TextSelection selection( bool ) const override;

    //* retrieve editor
    QWidget& editor() const override
    { return *editor_; }

    //* close button
    QAbstractButton& closeButton() const override
    { return *closeButton_; }

    //@}

    //*@name modifiers
    //@{

    //* string to find
    void setText( const QString& ) override;

    //* synchronize searched strings and ComboBox
    void synchronize() override;

    //@}

    public Q_SLOTS:

    //* take action when at least one match is found
    void matchFound() override;

    //* take action when no match is found
    void noMatchFound() override;

    private Q_SLOTS:

    //* create Selection object when find button is pressed
    void _find()
    { emit find( selection( false ) ); }

    //* create Selection object when find button is pressed
    void _findPrevious()
    {
        findBackward_ = true;
        emit find( selection( false ) );
    }

    //* create Selection object when find button is pressed
    void _findNext()
    {
        findBackward_ = false;
        emit find( selection( false ) );
    }

    //* create Selection object when find button is pressed
    void _findNoIncrement()
    { emit find( selection( true ) ); }

    //* update button state depending on the string to find
    void _updateButtons( const QString& text = QString() );

    private:

    //* add button to disabled button list
    void _addDisabledButton( QAbstractButton* );

    //* editor
    LineEditor* editor_;

    //* backward search if checked
    bool findBackward_ = false;

    //* find button
    QAbstractButton* findNextButton_ = nullptr;

    //* find button
    QAbstractButton* findPreviousButton_ = nullptr;

    //* close button
    QAbstractButton* closeButton_ = nullptr;

    //* list of buttons to enable/disable depending of the editor text
    QList<QAbstractButton*> buttons_;

};

#endif
