/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "StoredPlaylistItem.h"

#include <QObject>
#include <QRegExp>

//__________________________________________________________________
const QString StoredPlaylistItem::MimeType = "musicbox/storedplaylist-item-list";

//__________________________________________________________________
StoredPlaylistItem::Type StoredPlaylistItem::parentType( StoredPlaylistItem::Type type )
{
    switch( type )
    {
        default:
        case Type::Invalid:
        case Type::Playlist:
        return Type::Invalid;

        case Type::Track: return Type::Playlist;
    }
}

//__________________________________________________________________
bool StoredPlaylistItem::isChild( const StoredPlaylistItem& other ) const
{

    switch( type_ )
    {
        default:
        case Type::Invalid:
        case Type::Playlist: return false;
        case Type::Track: return other.type_ == Type::Playlist && other.playlist_ == playlist_;
    }

}

//__________________________________________________________________
QString StoredPlaylistItem::displayName( const QString& value )
{
    QRegExp regExp( "(.*)_\\{\\d+\\}" );
    return value.indexOf( regExp ) >= 0 ? regExp.cap(1):value;
}


//__________________________________________________________________
TimeStamp StoredPlaylistItem::creation( const QString& value )
{
    QRegExp regExp( "_\\{(\\d+)\\}" );
    if( value.indexOf( regExp ) >= 0 ) return TimeStamp( regExp.cap(1).toInt() );
    else return TimeStamp();
}

//__________________________________________________________________
void StoredPlaylistItem::setPlaylist( const QString& value )
{
    playlist_ = value;
    displayName_ = displayName( value );
    creation_ = creation( value );
}

//__________________________________________________________________
bool operator == (const StoredPlaylistItem& lhs, const StoredPlaylistItem& rhs )
{
    if( lhs.type_ != rhs.type_ ) return false;
    else switch( lhs.type_ )
    {
        default: return false;
        case StoredPlaylistItem::Type::Playlist: return lhs.playlist_ == rhs.playlist_;
        case StoredPlaylistItem::Type::Track: return lhs.playlist_ == rhs.playlist_ && lhs.trackIndex() == rhs.trackIndex();
    }
}

//__________________________________________________________________
bool operator < (const StoredPlaylistItem& lhs, const StoredPlaylistItem& rhs )
{
    if( lhs.type() != rhs.type() )
    {

        if( lhs.type() == StoredPlaylistItem::Type::Track ) return true;
        else return false;

    } else  switch( lhs.type_ ) {

        default: return false;

        case StoredPlaylistItem::Type::Playlist: return lhs.playlist_ < rhs.playlist_;
        case StoredPlaylistItem::Type::Track:
        {
            if( lhs.playlist_ != rhs.playlist_ ) return lhs.playlist_ < rhs.playlist_;
            else return lhs.trackIndex() < rhs.trackIndex();
        }
    }

}
