#ifndef TagLibInterface_h
#define TagLibInterface_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "TrackInformation.h"

#if WITH_TAGLIB
#include <taglib/fileref.h>
#endif

class TagLibInterface: private Base::Counter<TagLibInterface>
{

    public:

    //* constructor
    explicit TagLibInterface():
        Counter( "TagLibInterface" )
    {}

    //* constructor
    explicit TagLibInterface( const File& );

    //*@name accessors
    //@{

    //* true if valid
    bool isValid() const;

    //* get track information from file
    TrackInformation trackInformation( TrackInformation::Mask = TrackInformation::All ) const;

    //@}

    //*@name modifier
    //@{

    //* assign file
    void setFile( const File& );

    //* assign track information to file
    bool setTrackInformation( const TrackInformation&, TrackInformation::Mask = TrackInformation::All );

    //@}

    private:

    //* file
    File file_;

    #if WITH_TAGLIB
    //* taglib fileref
    TagLib::FileRef fileRef_;
    #endif


};


#endif
