/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ToolTipWidget.h"

#include "Debug.h"
#include "GridLayout.h"
#include "GridLayoutItem.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QLayout>

//_______________________________________________________
ToolTipWidget::ToolTipWidget( QWidget* parent ):
    BaseToolTipWidget( parent ),
    pixmapSize_( 96 ),
    mask_( Default )
{

    Debug::Throw( "ToolTipWidget::ToolTipWidget.\n" );

    setFollowMouse( true );

    // layout
    auto hLayout = new QHBoxLayout;
    hLayout->setMargin( 10 );
    hLayout->setSpacing( 10 );
    setLayout( hLayout );

    hLayout->addWidget( iconLabel_ = new QLabel( this ) );
    iconLabel_->setAlignment( Qt::AlignHCenter|Qt::AlignTop );

    auto vLayout = new QVBoxLayout;
    vLayout->setMargin( 0 );
    vLayout->setSpacing( 5 );
    hLayout->addLayout( vLayout );

    // grid layout
    vLayout->addStretch( 1 );
    auto gridLayout = new GridLayout;
    gridLayout->setMaxCount( 2 );
    gridLayout->setColumnAlignment( 0, Qt::AlignVCenter|Qt::AlignRight );
    gridLayout->setColumnAlignment( 1, Qt::AlignVCenter|Qt::AlignLeft );
    gridLayout->setMargin( 0 );
    gridLayout->setSpacing( 5 );
    vLayout->addLayout( gridLayout );

    // items
    ( artistItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Artist:" ) );
    ( albumItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Album:" ) );
    ( titleItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Title:" ) );
    ( yearItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Year:" ) );
    ( discItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Disc:" ) );
    ( trackItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Track:" ) );
    ( durationItem_ = new GridLayoutItem( this, gridLayout ) )->setKey( tr( "Duration:" ) );

    // add stretch
    vLayout->addStretch( 1 );

    // configuration
    connect( Base::Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );

}

//_____________________________________________
bool ToolTipWidget::updateCover( const CoverManager::CoverList& coverList )
{
    return std::any_of( coverList.begin(), coverList.end(),
        [&]( const CoverManager::CoverPair& pair )
        { return _updateCover( pair.first, pair.second ); } );
}

//_____________________________________________
void ToolTipWidget::_updateConfiguration()
{
    Debug::Throw( "ToolTipWidget::_updateConfiguration.\n" );
    if( XmlOptions::get().contains( "TOOLTIPS_PIXMAP_SIZE" ) ) setPixmapSize( XmlOptions::get().get<int>( "TOOLTIPS_PIXMAP_SIZE" ) );
    if( XmlOptions::get().contains( "TOOLTIPS_MASK" ) ) setMask( static_cast<Types>( XmlOptions::get().get<int>( "TOOLTIPS_MASK" ) ) );
}

