#ifndef TrackInformationModel_h
#define TrackInformationModel_h

/******************************************************************************
*
* Copyright (C) 2017 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "CoverCache.h"
#include "Debug.h"
#include "ListModel.h"
#include "MpdDefines.h"
#include "TrackInformation.h"

#include <QIcon>
#include <QMimeData>


class TrackInformationModel: public ListModel<TrackInformation, TrackInformation::WeakEqualFTor>, private Base::Counter<TrackInformationModel>
{

    Q_OBJECT

    public:

    //* constructor
    explicit TrackInformationModel( QObject* = nullptr );

    //*@name accessors
    //@{

    //* flags
    Qt::ItemFlags flags( const QModelIndex& ) const override;

    //* return data
    QVariant data( const QModelIndex&, int ) const override;

    //* header data
    QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole) const override
    { return QVariant(); }

    //* number of columns for a given index
    int columnCount( const QModelIndex& = QModelIndex() ) const override
    { return 1; }

    //* current track index
    virtual QModelIndex currentTrackIndex() const
    { return QModelIndex(); }

    //* state icon
    virtual QIcon stateIcon() const
    { return QIcon(); }

    //* get icon from collection item
    QIcon icon( const TrackInformation& ) const;

    //@}

    //*@name modifiers
    //@{

    void setCoverCache( const CoverCache::Pointer& );

    //@}

    private:

    //* cover cache
    CoverCache::Pointer cache_;

};

#endif
